require 'spec_helper'

describe Endicia::LabelResponse do
  describe "initialize" do
    let(:result) {
      OpenStruct.new(
        "LabelRequestResponse"  => {
          image_key             => image_value,
          "Status"              => Faker::Lorem.sentence,
          "TrackingNumber"      => Faker::Lorem.sentence,
          "FinalPostage"        => Faker::Lorem.sentence,
          "TransactionDateTime" => Faker::Lorem.sentence,
          "TransactionId"       => Faker::Lorem.sentence,
          "PostmarkDate"        => Faker::Lorem.sentence,
          "PostageBalance"      => Faker::Lorem.sentence,
          "Pic"                 => Faker::Lorem.sentence,
          "ErrorMessage"        => Faker::Lorem.sentence,
          "ReferenceId"         => Faker::Lorem.sentence,
          "ReferenceId2"        => Faker::Lorem.sentence,
          "ReferenceId3"        => Faker::Lorem.sentence,
          "ReferenceId4"        => Faker::Lorem.sentence,
          "RequesterId"         => Faker::Lorem.sentence,
          "CostCenter"          => Faker::Lorem.sentence,
          "RequestBody"         => Faker::Lorem.sentence,
          "RequestUrl"          => Faker::Lorem.sentence,
          "Ignored"             => Faker::Lorem.sentence,
        },
        body: image_body
      )
    }
    let(:image_body) { "<Image>ksadjhfalskjfhalsjfhaslfjhasdljkf</Image>" }
    let(:expected_image_value) { Faker::Lorem.sentence }
    let(:different_image_value) { "a throwaway addendum" }
    let(:image_key) { "Base64LabelImage" }
    let(:image_value) { expected_image_value }

    subject(:response) { described_class.new(result) }

    it "turns the awful Enterprise keys into snake-case and sets 'em up appropriate" do
      expect(response.status).to eq(result["LabelRequestResponse"]["Status"])
      expect(response.tracking_number).to eq(result["LabelRequestResponse"]["TrackingNumber"])
      expect(response.final_postage).to eq(result["LabelRequestResponse"]["FinalPostage"])
      expect(response.transaction_date_time).to eq(result["LabelRequestResponse"]["TransactionDateTime"])
      expect(response.transaction_id).to eq(result["LabelRequestResponse"]["TransactionId"])
      expect(response.postmark_date).to eq(result["LabelRequestResponse"]["PostmarkDate"])
      expect(response.postage_balance).to eq(result["LabelRequestResponse"]["PostageBalance"])
      expect(response.pic).to eq(result["LabelRequestResponse"]["Pic"])
      expect(response.error_message).to eq(result["LabelRequestResponse"]["ErrorMessage"])
      expect(response.reference_id).to eq(result["LabelRequestResponse"]["ReferenceId"])
      expect(response.reference_id2).to eq(result["LabelRequestResponse"]["ReferenceId2"])
      expect(response.reference_id3).to eq(result["LabelRequestResponse"]["ReferenceId3"])
      expect(response.reference_id4).to eq(result["LabelRequestResponse"]["ReferenceId4"])
      expect(response.requester_id).to eq(result["LabelRequestResponse"]["RequesterId"])
      expect(response.cost_center).to eq(result["LabelRequestResponse"]["CostCenter"])
      expect(response.request_body).to eq(result["LabelRequestResponse"]["RequestBody"])
      expect(response.request_url).to eq(result["LabelRequestResponse"]["RequestUrl"])
    end

    it "masks data in the body" do
      expect(response.response_body).to eq("<Image>[data]</Image>")
    end

    context "for typical case" do
      let(:image_key) { "Base64LabelImage" }
      let(:image_value) { expected_image_value }

      it "translates Base64LabelImage to 'image'" do
        expect(response.image).to eq(expected_image_value)
      end
    end

    context "for customs form case" do
      context "when there is only one Image part" do
        let(:image_key) { "Label" }
        let(:image_value) {
          {
            "Image" => {
              "__content__" => expected_image_value
            }
          }
        }

        it "translates Base64LabelImage to 'image'" do
          expect(response.image).to eq(expected_image_value)
        end

        it "masks data in the body" do
          expect(response.response_body).to eq("<Image>[data]</Image>")
        end
      end

      context "when there are multiple Image parts" do
        let(:image_body) { '<Image PartNumber="1">ksadjhfalskjfhalsjfhaslfjhasdljkf</Image><Image PartNumber="2">taoenusaoetuhnoaeuteoa</Image>' }
        let(:image_key) { "Label" }
        let(:image_value) {
          {
            "Image" => [
              {
                "__content__" => expected_image_value,
              },
              {
                "__content__" => different_image_value,
              },
            ],
          }
        }

        it "translates the first Image part to 'image'" do
          expect(response.image).to eq(expected_image_value)
        end

        it "masks data in the body" do
          expect(response.response_body).to eq('<Image PartNumber="1">[data]</Image><Image PartNumber="2">[data]</Image>')
        end
      end
    end
  end

  describe "#label_generated?" do
    let(:status) { nil }
    subject(:label_response) { described_class.new(OpenStruct.new(body: "<foo></foo>", LabelRequestResponse: {"Status" => status})) }

    context "when '0'" do
      let(:status) { "0" }
      specify { expect(label_response.label_generated?).to eq(true) }
    end

    context "when 0" do
      let(:status) { 0 }
      specify { expect(label_response.label_generated?).to eq(true) }
    end

    context "when not 0" do
      let(:status) { "1234" }
      specify { expect(label_response.label_generated?).to eq(false) }
    end
  end

  describe "#critical_error?" do
    let(:status) { nil }
    subject(:label_response) { described_class.new(OpenStruct.new(body: "<foo></foo>", LabelRequestResponse: {"Status" => status})) }

    context "when 0" do
      let(:status) { 0 }
      specify { expect(label_response.critical_error?).to eq(false) }
    end

    context "when not 1234" do
      let(:status) { "1234" }
      specify { expect(label_response.critical_error?).to eq(false) }
    end
    [
      1101,
      2011,
      2012,
      2013,
      2014,
      2015,
      2020,
      2021,
      12101,
      12103,
      12104,
      12115,
      12116,
      12117,
      12120,
      12130,
      12133,
      12503,
      12505,
      12507,
      12515,
      12525,
      100201,
    ].each do |code|
      context "when #{code}" do
        let(:status) { code.to_s }
        specify { expect(label_response.critical_error?).to eq(true) }
      end
    end
  end
 
end
